package sample.web;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Servlet implementation class TableDataServlet
 */
@WebServlet("/TableDataServlet")
public class TableDataServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;

	/**
	 * @see HttpServlet#service(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		List<Map<String, Object>> week = getOneWeek();

		request.setAttribute("week", week);

		// JSPへ処理を遷移させる
		request.getRequestDispatcher("/jsp/schedule.jsp").forward(request, response);

	}

	private List<Map<String, Object>> getOneWeek() {
		// 現在日付を取得
		Calendar cal = GregorianCalendar.getInstance();

		// 現在が何曜日であるかを取得する。日曜日：1、月曜日：2…、土曜日：7
		int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);

		// 現在日付が含まれる１週間の日曜日を計算し、先程取得した日付を日曜日へ移動する。
		// 曜日の値は[日曜日=1]なので日曜日=現在日付の日にち-( 曜日-1 )
		cal.add(Calendar.DATE, -(dayOfWeek - 1));

		// 1週間分のデータを作成する。1日はMap形式のデータに格納され、それを束ねて1週間を1つのListとする
		// List<Map<String,Object>> week = new ArrayList<Map<String,Object>>();
		List<Map<String, Object>> week = new ArrayList<Map<String, Object>>();

		for ( int i=0;i<7;i++) {
			Map<String,Object> map = new HashMap<String,Object>();

			// 曜日ごとにCalendarを作成する。日曜日、月曜日…土曜日の順。
			Calendar oneDate = Calendar.getInstance();
			oneDate.setTime(cal.getTime());	// 現在日付から計算した日曜日を設定
			oneDate.add(Calendar.DATE, i);	// 日曜日＋ループの日数を足す

			// mapに格納する情報を作る
			map.put("date", oneDate.getTime());

			// 日付の文字列表現
			int month = oneDate.get(Calendar.MONTH) +1;	// 月
			int date = oneDate.get(Calendar.DATE);	// 日

			// スケジュールの文字列を作成。ここでは暫定的に日付＋固定文字列
			String schedule = String.valueOf(month) + "/" + String.valueOf(date) + "のスケジュール。";
			map.put("schedule", schedule);

			// 戻りのListへ格納
			week.add(map);
		}

		return week;
	}

}
