/**
 *
 */
package sample.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.naming.InitialContext;
import javax.sql.DataSource;

import sample.entity.Todo;

/**
 * @author A-pZ
 *
 */
public class DatabaseConnection {
    private Connection connection = null;

    public List<Todo> select() throws Exception {
        // 実行するSQLを作成
        String sql = "SELECT title , task , limitdate , lastupdate , userid, status "
                + "FROM todo_list WHERE userid like CONCAT('%',?,'%')";

        // ステートメント(SQLを格納・実行するコンテナ)を取得。
        // 動的SQLであれば取得するステートメントはPreparedStatementにすること。
        // Statement statement = connection.createStatement();
        PreparedStatement statement = connection.prepareStatement(sql);

        // PreparedStatementのプレースホルダーに変数を順番に格納する。
        statement.setString(1, "USER");

        ResultSet rs = statement.executeQuery();

        List<Todo> resultList = new ArrayList<Todo>();

        while (rs.next()) {
            // 検索結果1レコードの内容を取得する
            Todo todo = new Todo();
            todo.setTitle(rs.getString("title"));
            todo.setTask(rs.getString("task"));
            todo.setLimitdate(rs.getTimestamp("limitdate"));
            todo.setLastupdate(rs.getTimestamp("lastupdate"));
            todo.setUserid(rs.getString("userid"));
            todo.setStatus(rs.getInt("status"));

            // 検索結果をListへ格納
            resultList.add(todo);
        }

        return resultList;
    }

    public void insert() throws Exception {

        // 実行するSQLを作成
        String sql = "INSERT INTO `sample`.`todo_list` ( `title`,`task`, `limitdate`, `lastupdate`, `userid`, `status` "
                + ") VALUES ( ? , ? , ? , now() , ? , ? );";

        try {
            // ステートメント(SQLを格納・実行するコンテナ)を取得。
            PreparedStatement statement = connection.prepareStatement(sql);

            // PreparedStatementのプレースホルダーに変数を順番に格納する。
            statement.setString(1, "講習会準備");
            statement.setString(2, "講習会のスライド作成、配布ソースコードの準備");
            statement.setString(3, "2012-06-12");
            statement.setString(4, "USER99");
            statement.setInt(5, 0);

            // 更新系操作はexecuteUpdate()を利用し、その対象件数を取得することができる。
            int count = statement.executeUpdate();
            System.out.println("1つ目の追加:" + count);

            // PreparedStatementのプレースホルダーに変数を順番に格納する。
            statement.setString(1, "講習会");
            statement.setString(2, "2012年7月の講習会を実施");
            statement.setString(3, "2012-07-01");
            statement.setString(4, "USER99");
            statement.setInt(5, 0);

            // 更新系操作はexecuteUpdate()を利用し、その対象件数を取得することができる。
            count = statement.executeUpdate();
            System.out.println("2つ目の追加:" + count);

            // トランザクションをコミットする。
            connection.commit();
        } catch (Exception e) {
            connection.rollback();
            throw e;
        }
    }

    /**
     * データベースとの接続を取得する。 もし取得していた場合には既存の接続を利用し、 取得していない場合は新たにコンテナから取得する。
     *
     * @return
     * @throws Exception
     */
    public Connection getConnection() throws Exception {

        // NamingException, SQLExceptionがスローされる
        try {
            if (connection == null || connection.isClosed()) {
                // JNDI参照コンテキストを取得
                InitialContext initCtx = new InitialContext();

                // Tomcatで管理されたデータベース接続をJNDI経由で取得。
                // java:comp/env/を必ずつける。
                DataSource ds = (DataSource) initCtx
                        .lookup("java:comp/env/jdbc/localDB");

                // データベース接続を取得する
                connection = ds.getConnection();

                // 自動コミットを行わず、
                // 更新系処理では常にトランザクション管理を行うように設定する。
                connection.setAutoCommit(false);

                // 接続が正しく完了するとコンソールにメッセージを出力
                System.out.println("接続を開きました");
            }
        } catch (Exception e) {
            // もし接続取得で例外が出た場合はconnection=nullにし、
            // 発生した例外はそのまま送出する。
            e.printStackTrace();
            connection = null;
            throw e;
        }

        return connection;
    }

    /**
     * 接続を閉じる。 確実に接続を解放するためfinallyでconnection=nullを行う。
     */
    public void closeConnection() {
        try {
            connection.close();
            // 接続が正しく閉じられるとコンソールにメッセージを出力
            System.out.println("接続を閉じました");
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            connection = null;
        }
    }
}
