package sample.dao;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;

import sample.entity.Todo;

public class TodoDAO extends MySQLDAO {
    public List<Todo> select() throws Exception {
        // 実行するSQLを作成
        String sql = "SELECT title , task , limitdate , lastupdate , userid, status "
                   + "FROM todo_list WHERE userid like CONCAT('%',?,'%')";

        // ステートメント(SQLを格納・実行するコンテナ)を取得。
        // 動的SQLであれば取得するステートメントはPreparedStatementにすること。
        // Statement statement = connection.createStatement();
        PreparedStatement statement = connection.prepareStatement(sql);

        // PreparedStatementのプレースホルダーに変数を順番に格納する。
        statement.setString(1, "USER");

        ResultSet rs = statement.executeQuery();

        List<Todo> resultList = new ArrayList<Todo>();

        while (rs.next()) {
            // 検索結果1レコードの内容を取得する
            Todo todo = new Todo();
            todo.setTitle( rs.getString("title"));
            todo.setTask( rs.getString("task"));
            todo.setLimitdate( rs.getTimestamp("limitdate"));
            todo.setLastupdate( rs.getTimestamp("lastupdate"));
            todo.setUserid( rs.getString("userid"));
            todo.setStatus( rs.getInt("status"));

            // 検索結果をListへ格納
            resultList.add(todo);
        }

        return resultList;
    }

    public void insert() throws Exception {

        // 実行するSQLを作成
        String sql = "INSERT INTO `sample`.`todo_list` ( `title`,`task`, `limitdate`, `lastupdate`, `userid`, `status` "
                + ") VALUES ( ? , ? , ? , now() , ? , ? );";

        try {
            // ステートメント(SQLを格納・実行するコンテナ)を取得。
            // 動的SQLであれば取得するステートメントはPreparedStatementにすること。
            // Statement statement = connection.createStatement();
            PreparedStatement statement = connection.prepareStatement(sql);

            // PreparedStatementのプレースホルダーに変数を順番に格納する。
            statement.setString(1, "講習会準備");
            statement.setString(2, "講習会のスライド作成、配布ソースコードの準備");
            statement.setString(3, "2012-06-12");
            statement.setString(4, "USER99");
            statement.setInt(5, 0);

            // 更新系操作はexecuteUpdate()を利用し、その対象件数を取得することができる。
            int count = statement.executeUpdate();
            System.out.println("1つ目の追加:" + count);

            // PreparedStatementのプレースホルダーに変数を順番に格納する。
            statement.setString(1, "講習会");
            statement.setString(2, "2012年7月の講習会を実施");
            statement.setString(3, "2012-07-01");
            statement.setString(4, "USER99");
            statement.setInt(5, 0);

            // 更新系操作はexecuteUpdate()を利用し、その対象件数を取得することができる。
            count = statement.executeUpdate();
            System.out.println("2つ目の追加:" + count);

            // トランザクションをコミットする。
            connection.commit();
        } catch (Exception e) {
            connection.rollback();
            throw e;
        }
    }
}
